--[[
    Aaron MacDonald
    August 28, 2010
]]

require('graphics')
require('constants')
require('utility')

local SPRITE_NORTH = '_n'
local SPRITE_EAST = '_e'
local SPRITE_SOUTH = '_s'
local SPRITE_WEST = '_w'

actor = {}

function actor.diagonalSpeed(speed)
    return utility.roundNumber(
      (speed * speed) / math.sqrt((speed * speed) * 2) )
end

function actor.newActor(actorType, x, y, facing)
    local sprite = actorType.sprite

    local act = {
        type = actorType,
        spriteSet = { [constants.UP] = graphics.loadImage(sprite
                        .. SPRITE_NORTH),
                      [constants.RIGHT] = graphics.loadImage(sprite
                        .. SPRITE_EAST),
                      [constants.DOWN] = graphics.loadImage(sprite
                        .. SPRITE_SOUTH),
                      [constants.LEFT] = graphics.loadImage(sprite
                        .. SPRITE_WEST) },

        x = x,
        y = y,
        facing = facing
    }

    return act
end

function actor.getVelocity(act, movementControls)
    local dx = 0
    local dy = 0

    local speed = act.type.speed
    local diagonalSpeed = act.type.diagonalSpeed

    if movementControls[constants.UP] then
        dy = dy - speed
    end

    if movementControls[constants.DOWN] then
        dy = dy + speed
    end

    if movementControls[constants.RIGHT] then
        dx = dx + speed
    end

    if movementControls[constants.LEFT] then
        dx = dx - speed
    end

    if dx == 0 then
        if dy > 0 then
            act.facing = constants.DOWN
        elseif dy < 0 then
            act.facing = constants.UP
        end
    end

    if dy == 0 then
        if dx > 0 then
            act.facing = constants.RIGHT
        elseif dx < 0 then
            act.facing = constants.LEFT
        end
    end

    if (dx ~= 0) and (dy ~= 0) then
        dx = diagonalSpeed * utility.signedComponent(dx)
        dy = diagonalSpeed * utility.signedComponent(dy)
    end

    return dx, dy
end

function actor.move(act, dx, dy, timeX, timeY)
    local time = time or 1.0

    act.x = act.x + math.floor(dx * timeX)
    act.y = act.y + math.floor(dy * timeY)
end

function actor.draw(act)
    love.graphics.draw(act.spriteSet[act.facing],
      act.x - act.type.spriteOffsetX, act.y - act.type.spriteOffsetY)
end